fs = require "fs"
pathLib = require "path"
_ = require "underscore"

require "./dk-template-ejs"
require "./dk-template-eco"
require "./dk-template-mustache"
require "./dk-template-layout"
require "./dk-template-helpers"

dk.plugin "template", ["assets"], ->
  template = {}
  rootDir = "#{process.cwd()}/views"
  renderers = {}
  preprocessors = {}

  # PUBLIC METHODS

  template.config = (opts) ->
    rootDir = opts.rootDir if opts.dir

  template.render = (path, locals={}) ->
    markup = null
    thisSuffix = null
    unless /^\//.test path
      path = "#{rootDir}/#{path}"

    _.each _.keys(renderers), (suffix) ->
      fullPath = "#{path}.#{suffix}"
      if pathLib.existsSync(fullPath)
        markup = fs.readFileSync(fullPath, 'utf8')
        thisSuffix = suffix
    renderTemplate markup, thisSuffix, locals

  template.addRenderer = (suffix, func) ->
    renderers[suffix] = func

  template.addPreprocessor = (suffix, func) ->
    preprocessors[suffix] = func

  template.run = ->
    wireAssets dk.assets

  # PRIVATE METHODS

  wireAssets = (assets) ->
    assets.package "/js/dk-template.js",
                   "#{__dirname}/../public/js/dk-template.coffee"
    assets.package "/js/dk/template/all-templates.js", buildAllTemplates

  preprocess = (markup, suffix) ->
    preprocessor = preprocessors[suffix] || (markup) -> markup
    preprocessor markup

  renderTemplate = (markup, suffix, locals={}) ->
    allLocals = _.clone template.helpers.all()
    _.extend(allLocals, locals)
    markup = preprocess markup, suffix
    renderers[suffix](markup, allLocals)

  buildAllTemplates = ->
    allTemplates = {}
    # FIXME: Make recursive
    fs.readdirSync(rootDir).forEach (filename) ->
      key = pathLib.basename(filename, pathLib.extname(filename))
      suffix = pathLib.extname(filename).replace('.', '')

      allTemplates[key] = {}
      allTemplates[key].renderer = suffix
      allTemplates[key].template = preprocess fs.readFileSync("#{rootDir}/#{filename}").toString(), suffix
    return "dk.plugin('template.allTemplates', ['template'], function() {return #{JSON.stringify allTemplates};});"

  return template
